﻿Public Class Form1

    ' Declare the Sleep function from Kernel32.dll
    Private Declare Sub Sleep Lib "Kernel32" (ByVal dwMilliseconds As Integer)

    ' Arrays to hold checkbox controls for AI channels
    Dim checkboxAi(0 To 23) As CheckBox
    ' Arrays to hold textbox controls for AI channel values
    Dim textboxAiValue(0 To 23) As TextBox
    ' Arrays to hold checkbox controls for DI channels
    Dim checkboxDi(0 To 3) As CheckBox
    ' Arrays to hold checkbox controls for DO channels
    Dim checkboxDo(0 To 3) As CheckBox

    ' Index of the selected device
    Dim DevIndex As Integer
    ' Flag to indicate if the device is running
    Dim bRun As Boolean
    ' Array to hold the device model information
    Dim Model(4) As Byte


    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        Dim temp As Integer
        Dim i As Integer

        ' Find the USB 3 DAQ device and get the number of available devices
        temp = FindUSB3DAQ()
        If (temp > 0) Then
            ' Add each available device index to the combo box
            For i = 0 To temp - 1
                comboboxDeviceIndex.Items.Add(i)
            Next
            ' Select the first device index in the combo box
            comboboxDeviceIndex.SelectedIndex = 0
            ' Enable the start button
            btnStart.Enabled = True
        Else
            ' Disable the start button if no device is found
            btnStart.Enabled = False
        End If

        ' Create and configure checkbox and textbox controls for AI channels
        For i = 0 To 23
            checkboxAi(i) = New CheckBox
            checkboxAi(i).Width = 60
            checkboxAi(i).Text = "Ai " & i.ToString
            checkboxAi(i).Left = 20 + 75 * (i \ 8)
            checkboxAi(i).Top = 20 + (i Mod 8) * 20
            GroupBox1.Controls.Add(checkboxAi(i))

            textboxAiValue(i) = New TextBox
            textboxAiValue(i).Width = 50
            textboxAiValue(i).Left = 20 + 55 * (i Mod 8)
            textboxAiValue(i).Top = 20 + (i \ 8) * 60
            GroupBox2.Controls.Add(textboxAiValue(i))
        Next

        ' Check the first 8 AI channels by default
        For i = 0 To 7
            checkboxAi(i).Checked = True
        Next

        ' Initialize the textboxes for AO values
        textboxAO_0.Text = 0
        textboxAO_1.Text = 0
        textboxAO_2.Text = 0
        textboxAO_3.Text = 0

        ' Create and configure checkbox controls for DI channels
        For i = 0 To 3
            checkboxDi(i) = New CheckBox
            checkboxDi(i).Text = "Di " & i.ToString
            checkboxDi(i).Left = 54
            checkboxDi(i).Top = 20 + 22 * i
            GroupBox4.Controls.Add(checkboxDi(i))
        Next

        ' Create and configure checkbox controls for DO channels
        For i = 0 To 3
            checkboxDo(i) = New CheckBox
            checkboxDo(i).Text = "Do " & i.ToString
            checkboxDo(i).Left = 54
            checkboxDo(i).Top = 20 + 22 * i
            GroupBox5.Controls.Add(checkboxDo(i))
        Next

        ' Disable the stop button initially
        btnStop.Enabled = False
        ' Set the running flag to false
        bRun = False

    End Sub

    Private Sub btnStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStart.Click

        Dim i As Integer
        Dim j As Integer
        Dim temp As Integer
        Dim SN(8) As Byte

        ' Get the selected device index from the combo box
        DevIndex = comboboxDeviceIndex.SelectedIndex

        ' Open the device with the selected index
        temp = USB3OpenDevice(DevIndex)

        ' Get and display the device serial number
        temp = USB3GetDeviceSN(DevIndex, SN(0))
        textboxSN.Text = "SN: " + System.Text.Encoding.ASCII.GetString(SN)

        ' Get the device model information
        temp = USB3GetDeviceModel(DevIndex, Model(0))
        textboxModel.Text = "Model: " + System.Text.Encoding.ASCII.GetString(Model)

        If Model(1) = &H31 Or Model(1) = &H32 Then
            ' If the model is 31xx or 32xx, then it has AI function

            '-----------------------------------------
            ' AI related settings

            ' Configure each of the 24 AI channels
            For i = 0 To 23
                ' Select whether to enable the channel
                temp = SetUSB3AiChanSel(DevIndex, i, CByte(checkboxAi(i).CheckState))
                ' Set the range of the channel
                temp = SetUSB3AiRange(DevIndex, i, 10.24)

                ' Clear the AI value display for all channels
                textboxAiValue(i).Text = ""
            Next

            ' Set the acquisition mode to OneShot mode
            temp = SetUSB3AiSampleMode(DevIndex, 1)

            ' Set the number of points to acquire per channel in OneShot mode to 200
            temp = SetUSB3AiOneShotPoints(DevIndex, 200)

            ' Set the AI input connection type to NRSE
            temp = SetUSB3AiConnectType(DevIndex, 1)

            ' Set the AI input sampling period to 10000 ns (10us), with a sampling rate of 100kHz
            temp = SetUSB3AiSampleRate(DevIndex, 10000)

            ' Set the AI trigger source to software trigger
            temp = SetUSB3AiTrigSource(DevIndex, 0)

            ' Set the AI sampling clock to the internal AI sampling clock
            temp = SetUSB3AiConvSource(DevIndex, 0)

            ' Set the AI pre-trigger points to 0
            temp = SetUSB3AiPreTrigPoints(DevIndex, 0)

            ' Clear the AI FIFO once
            temp = SetUSB3ClrAiFifo(DevIndex)

        End If

        If Model(1) = &H31 Or Model(1) = &H32 Then
            ' If the model is 31xx or 32xx, then it has DI function

            '----------------------------------------------
            ' DI related settings

            ' Set the acquisition mode to OneShot mode
            temp = SetUSB3DiSampleMode(DevIndex, 1)

            ' Set the number of points to acquire per channel in OneShot mode to 1
            temp = SetUSB3DiOneShotPoints(DevIndex, 1)

            ' Set the DI input sampling period to 10000 ns (10us), with a sampling rate of 100kHz
            temp = SetUSB3DiSampleRate(DevIndex, 10000)

            ' Set the DI trigger source to software trigger
            temp = SetUSB3DiTrigSource(DevIndex, 0)

            ' Set the DI sampling clock to the internal DI sampling clock
            temp = SetUSB3DiConvSource(DevIndex, 0)

            ' Set the DI pre-trigger points to 0
            temp = SetUSB3DiPreTrigPoints(DevIndex, 0)

            ' Clear the DI FIFO once
            temp = SetUSB3ClrDiFifo(DevIndex)

        End If

        ' Start Timer1
        Timer1.Enabled = True
        btnStart.Enabled = False
        btnStop.Enabled = True
        bRun = True

    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick

        Dim i As Integer
        Dim j As Integer
        Dim temp As Integer
        Static Dim ai(4800000) As Single
        Dim di(100) As Byte
        Dim average As Single

        If Model(1) = &H31 Or Model(1) = &H32 Then
            ' If the model is 31xx or 32xx, then it has AI function

            '------------------------------------------------
            ' Analog Input related functions

            ' Trigger a software trigger
            temp = SetUSB3AiSoftTrig(DevIndex)
            Sleep(10)
            ' Read 200 numbers from each channel and store them in the ai array
            temp = USB3GetAi(DevIndex, 200, ai(0), 1000)

            temp = 0
            ' Calculate the total number of selected channels in this acquisition
            For i = 0 To 23
                temp = temp + CByte(checkboxAi(i).CheckState)
            Next

            ' Calculate and display the average voltage measurement value for each channel
            For j = 0 To temp - 1
                average = 0
                For i = 0 To 199
                    average = average + ai(j * 200 + i)
                Next
                average = average / 200
                textboxAiValue(j).Text = Format(average, "0.0000")
            Next

            ' Clear the hardware trigger flag
            temp = SetUSB3ClrAiTrigger(DevIndex)
            ' Clear the AI FIFO
            temp = SetUSB3ClrAiFifo(DevIndex)

        End If

        If Model(1) = &H31 Or Model(1) = &H35 Then
            ' If the model is 31xx or 35xx, then it has AO function

            '---------------------------------------------
            ' Analog Output related functions

            If Val(textboxAO_0.Text) < -10 Or Val(textboxAO_0.Text) > 10 Then
                textboxAO_0.Text = 0
            End If
            ' Set the AO 0 voltage
            temp = SetUSB3AoImmediately(DevIndex, 0, CSng(Val(textboxAO_0.Text)))

            If Val(textboxAO_1.Text) < -10 Or Val(textboxAO_1.Text) > 10 Then
                textboxAO_1.Text = 0
            End If
            ' Set the AO 1 voltage
            temp = SetUSB3AoImmediately(DevIndex, 1, CSng(Val(textboxAO_1.Text)))

            If Val(textboxAO_2.Text) < -10 Or Val(textboxAO_2.Text) > 10 Then
                textboxAO_2.Text = 0
            End If
            ' Set the AO 2 voltage
            temp = SetUSB3AoImmediately(DevIndex, 2, CSng(Val(textboxAO_2.Text)))

            If Val(textboxAO_3.Text) < -10 Or Val(textboxAO_3.Text) > 10 Then
                textboxAO_3.Text = 0
            End If
            ' Set the AO 3 voltage
            temp = SetUSB3AoImmediately(DevIndex, 3, CSng(Val(textboxAO_3.Text)))

        End If

        If Model(1) = &H31 Or Model(1) = &H32 Then
            ' If the model is 31xx or 32xx, then it has DiDo function

            '----------------------------------------------------
            ' Digital Input related functions

            ' Trigger a software trigger
            temp = SetUSB3DiSoftTrig(DevIndex)

            ' Read the Di status and store it in the di array
            temp = USB3GetDi(DevIndex, 1, di(0), 100)

            ' Display the Di status
            ' The low four bits of di correspond to the four Di channels
            For i = 0 To 3
                checkboxDi(i).CheckState = di(0) And &H1
                di(0) = di(0) \ 2
            Next

            ' Clear the hardware trigger flag
            temp = SetUSB3ClrDiTrigger(DevIndex)

            ' Clear the Di FIFO
            temp = SetUSB3ClrDiFifo(DevIndex)


            '----------------------------------------------------
            ' Digital Output related functions

            For i = 0 To 3
                ' Set the Do output state
                temp = SetUSB3DoImmediately(DevIndex, i, checkboxDo(i).CheckState)
            Next

        End If

    End Sub

    Private Sub btnStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop.Click

        CloseDevice()

    End Sub

    Private Sub Form1_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing

        If bRun = True Then
            CloseDevice()
        End If

    End Sub

    Private Sub CloseDevice()

        Dim temp As Integer

        Timer1.Enabled = False
        Sleep(200)

        ' Close the device
        temp = USB3CloseDevice(DevIndex)

        btnStart.Enabled = True
        btnStop.Enabled = False
        bRun = False

    End Sub
End Class