﻿Public Class Form1

    ' Declare the Sleep function from Kernel32.dll
    Private Declare Sub Sleep Lib "Kernel32" (ByVal dwMilliseconds As Integer)

    ' Arrays to hold checkbox and textbox controls for AI channels
    Dim checkboxAi(0 To 23) As CheckBox
    Dim textboxAiValue(0 To 23) As TextBox
    ' Arrays to hold checkbox controls for DI and DO channels
    Dim checkboxDi(0 To 3) As CheckBox
    Dim checkboxDo(0 To 3) As CheckBox

    ' Index of the selected device
    Dim DevIndex As Integer
    ' Flag to indicate if the device is running
    Dim bRun As Boolean


    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        Dim temp As Integer
        Dim i As Integer

        ' Find the USB 3 DAQ device and get the number of available devices
        temp = FindUSB3DAQ()
        If (temp > 0) Then
            ' Add each available device index to the combo box
            For i = 0 To temp - 1
                comboboxDeviceIndex.Items.Add(i)
            Next
            ' Select the first device index in the combo box
            comboboxDeviceIndex.SelectedIndex = 0
            ' Enable the start button
            btnStart.Enabled = True
        Else
            ' Disable the start button if no device is found
            btnStart.Enabled = False
        End If

        ' Create and configure checkbox and textbox controls for AI channels
        For i = 0 To 23
            checkboxAi(i) = New CheckBox
            checkboxAi(i).Width = 60
            checkboxAi(i).Text = "Ai " & i.ToString
            checkboxAi(i).Left = 20 + 75 * (i \ 8)
            checkboxAi(i).Top = 20 + (i Mod 8) * 20
            GroupBox1.Controls.Add(checkboxAi(i))

            textboxAiValue(i) = New TextBox
            textboxAiValue(i).Width = 50
            textboxAiValue(i).Left = 20 + 55 * (i Mod 8)
            textboxAiValue(i).Top = 20 + (i \ 8) * 60
            GroupBox2.Controls.Add(textboxAiValue(i))
        Next

        ' Check the first 8 AI channels by default
        For i = 0 To 7
            checkboxAi(i).Checked = True
        Next

        ' Disable the stop button initially
        btnStop.Enabled = False
        ' Set the running flag to false
        bRun = False

    End Sub

    Private Sub btnStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStart.Click

        Dim i As Integer
        Dim j As Integer
        Dim temp As Integer
        Dim SN(8) As Byte

        ' Get the selected device index from the combo box
        DevIndex = comboboxDeviceIndex.SelectedIndex

        ' Open the device with the selected index
        temp = USB3OpenDevice(DevIndex)

        ' Get and display the device serial number
        temp = USB3GetDeviceSN(DevIndex, SN(0))
        textboxSN.Text = "SN: " + System.Text.Encoding.ASCII.GetString(SN)

        '-----------------------------------------
        ' AI related settings

        ' Configure each of the 24 AI channels
        For i = 0 To 23
            ' Select whether to enable the channel
            temp = SetUSB3AiChanSel(DevIndex, i, CByte(checkboxAi(i).CheckState))
            ' Set the range of the channel
            temp = SetUSB3AiRange(DevIndex, i, 10.24)

            ' Clear the AI value display for all channels
            textboxAiValue(i).Text = ""
        Next

        ' Set the acquisition mode to continuous acquisition
        temp = SetUSB3AiSampleMode(DevIndex, 0)

        ' Set the AI input connection type to NRSE
        temp = SetUSB3AiConnectType(DevIndex, 1)

        ' Set the AI input sampling period to 10000 ns (10us), with a sampling rate of 100kHz
        temp = SetUSB3AiSampleRate(DevIndex, 10000)

        ' Set the AI trigger source to software trigger
        temp = SetUSB3AiTrigSource(DevIndex, 0)

        ' Set the AI sampling clock to the internal AI sampling clock
        temp = SetUSB3AiConvSource(DevIndex, 0)

        ' Set the AI pre-trigger points to 0
        temp = SetUSB3AiPreTrigPoints(DevIndex, 0)

        ' Clear the AI FIFO once
        temp = SetUSB3ClrAiFifo(DevIndex)

        ' Trigger a software trigger once
        temp = SetUSB3AiSoftTrig(DevIndex)


        ' Start Timer1
        Timer1.Interval = 5
        Timer1.Enabled = True
        btnStart.Enabled = False
        btnStop.Enabled = True
        bRun = True

    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick

        Dim i As Integer
        Dim j As Integer
        Dim temp As Integer
        Static Dim ai(4800000) As Single
        Dim di(100) As Byte
        Dim average As Single

        '------------------------------------------------
        ' Analog Input related functions

        ' Read 1000 numbers from each channel in the AiFifo and store them in the ai array
        ' The data read out each time will be a set of continuously and uninterruptedly acquired data
        temp = USB3GetAi(DevIndex, 1000, ai(0), 1000)

        ' Display the number of data points remaining in the AiFifo that have not been read
        ' In long-term continuous and uninterrupted acquisition applications, pay attention to the change in the temp value
        ' If the temp value is gradually increasing, it means the data processing speed is not fast enough, resulting in data backlog in the AiFifo
        textboxFifoNum.Text = temp

        temp = 0
        ' Calculate the total number of selected channels in this acquisition
        For i = 0 To 23
            temp = temp + CByte(checkboxAi(i).CheckState)
        Next

        ' Calculate and display the average voltage measurement value for each channel
        For j = 0 To temp - 1
            average = 0
            For i = 0 To 999
                average = average + ai(j * 1000 + i)
            Next
            average = average / 1000
            textboxAiValue(j).Text = Format(average, "0.0000")
        Next

    End Sub

    Private Sub btnStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop.Click

        CloseDevice()

    End Sub

    Private Sub Form1_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing

        If bRun = True Then
            CloseDevice()
        End If

    End Sub

    Private Sub CloseDevice()

        Dim temp As Integer

        Timer1.Enabled = False
        Sleep(100)

        ' Clear the hardware trigger flag
        temp = SetUSB3ClrAiTrigger(DevIndex)
        ' Clear the AI FIFO
        temp = SetUSB3ClrAiFifo(DevIndex)

        ' Close the device
        temp = USB3CloseDevice(DevIndex)

        btnStart.Enabled = True
        btnStop.Enabled = False
        bRun = False

    End Sub
End Class