﻿Public Class Form1

    ' Declare the Sleep function from Kernel32.dll
    Private Declare Sub Sleep Lib "Kernel32" (ByVal dwMilliseconds As Integer)

    ' Arrays to hold checkbox controls for AI channels (although not fully utilized in this code for AI functionality)
    Dim checkboxAi(0 To 23) As CheckBox
    ' Arrays to hold textbox controls for AI channel values (although not fully utilized in this code for AI functionality)
    Dim textboxAiValue(0 To 23) As TextBox
    ' Arrays to hold checkbox controls for DI channels
    Dim checkboxDi(0 To 3) As CheckBox
    ' Arrays to hold checkbox controls for DO channels (not used in this code)
    Dim checkboxDo(0 To 3) As CheckBox

    ' Index of the selected device
    Dim DevIndex As Integer
    ' Flag to indicate if the device is running
    Dim bRun As Boolean


    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        Dim temp As Integer
        Dim i As Integer

        ' Find the USB 3 DAQ device and get the number of available devices
        temp = FindUSB3DAQ()
        If (temp > 0) Then
            ' Add each available device index to the combo box
            For i = 0 To temp - 1
                comboboxDeviceIndex.Items.Add(i)
            Next
            ' Select the first device index in the combo box
            comboboxDeviceIndex.SelectedIndex = 0
            ' Enable the start button
            btnStart.Enabled = True
        Else
            ' Disable the start button if no device is found
            btnStart.Enabled = False
        End If

        ' Create and configure checkbox controls for DI channels
        For i = 0 To 3
            checkboxDi(i) = New CheckBox
            checkboxDi(i).Text = "Di " + i.ToString
            checkboxDi(i).Top = 40
            checkboxDi(i).Left = 100 + i * 150
            GroupBox1.Controls.Add(checkboxDi(i))
        Next

        ' Disable the stop button initially
        btnStop.Enabled = False
        ' Set the running flag to false
        bRun = False

    End Sub

    Private Sub btnStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStart.Click

        Dim i As Integer
        Dim j As Integer
        Dim temp As Integer
        Dim SN(8) As Byte

        ' Get the selected device index from the combo box
        DevIndex = comboboxDeviceIndex.SelectedIndex

        ' Open the device with the selected index
        temp = USB3OpenDevice(DevIndex)

        ' Get and display the device serial number
        temp = USB3GetDeviceSN(DevIndex, SN(0))
        textboxSN.Text = "SN: " + System.Text.Encoding.ASCII.GetString(SN)

        '-----------------------------------------
        ' DI related settings

        ' Set the acquisition mode to continuous acquisition mode
        temp = SetUSB3DiSampleMode(DevIndex, 0)

        ' Set the DI input sampling period to 10000 ns (10us), with a sampling rate of 100kHz
        temp = SetUSB3DiSampleRate(DevIndex, 10000)

        ' Set the DI trigger source to software trigger
        temp = SetUSB3DiTrigSource(DevIndex, 0)

        ' Set the DI sampling clock to the internal DI sampling clock
        temp = SetUSB3DiConvSource(DevIndex, 0)

        ' Set the DI pre-trigger points to 0
        temp = SetUSB3DiPreTrigPoints(DevIndex, 0)

        ' Clear the DI FIFO once
        temp = SetUSB3ClrDiFifo(DevIndex)

        ' Trigger a software trigger once
        temp = SetUSB3DiSoftTrig(DevIndex)


        ' Start Timer1
        Timer1.Interval = 5
        Timer1.Enabled = True
        btnStart.Enabled = False
        btnStop.Enabled = True
        bRun = True

    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick

        Dim i As Integer
        Dim j As Integer
        Dim temp As Integer
        Dim di(5000) As Byte
        Dim average As Single

        '------------------------------------------------
        ' Digital Input related functions

        ' Read 5000 numbers from each channel in the DiFifo and store them in the di array
        ' The data read out each time will be a set of continuously and uninterruptedly acquired data
        temp = USB3GetDi(DevIndex, 5000, di(0), 1000)

        ' Display the number of data points remaining in the DiFifo that have not been read
        ' In long-term continuous and uninterrupted acquisition applications, pay attention to the change in the temp value
        ' If the temp value is gradually increasing, it means the data processing speed is not fast enough, resulting in data backlog in the DiFifo
        textboxFifoNum.Text = temp

        ' Display the latest state in the continuous DI data
        ' The low four bits of di correspond to the four Di channels
        For i = 0 To 3
            checkboxDi(i).CheckState = di(4999) And &H1
            di(4999) = di(4999) \ 2
        Next

    End Sub

    Private Sub btnStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop.Click

        CloseDevice

    End Sub

    Private Sub Form1_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing

        If bRun = True Then
            CloseDevice
        End If

    End Sub

    Private Sub CloseDevice()

        Dim temp As Integer

        Timer1.Enabled = False
        Sleep(100)

        ' Clear the hardware trigger flag
        temp = SetUSB3ClrDiTrigger(DevIndex)
        ' Clear the DI FIFO
        temp = SetUSB3ClrDiFifo(DevIndex)

        ' Close the device
        temp = USB3CloseDevice(DevIndex)

        btnStart.Enabled = True
        btnStop.Enabled = False
        bRun = False

    End Sub
End Class