﻿Public Class Form1

    ' Declare the Sleep function from Kernel32.dll
    Private Declare Sub Sleep Lib "Kernel32" (ByVal dwMilliseconds As Integer)

    ' Arrays to hold checkbox controls for AI channels (not fully utilized in this code for AI functionality)
    Dim checkboxAi(0 To 23) As CheckBox
    ' Arrays to hold textbox controls for AI channel values (not fully utilized in this code for AI functionality)
    Dim textboxAiValue(0 To 23) As TextBox
    ' Arrays to hold checkbox controls for DI channels (not used in this code)
    Dim checkboxDi(0 To 3) As CheckBox
    ' Arrays to hold checkbox controls for DO channels (not used in this code)
    Dim checkboxDo(0 To 3) As CheckBox

    ' Index of the selected device
    Dim DevIndex As Integer
    ' Flag to indicate if the device is running
    Dim bRun As Boolean

    ' Array to hold waveform data
    Dim WaveData(1000) As Single
    ' Variable to keep track of the number of rounds
    Dim Rounds As ULong
    ' Thread for downloading waveform data
    Dim threadDownload As Threading.Thread


    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        Dim temp As Integer
        Dim i As Integer

        ' Find the USB 3 DAQ device and get the number of available devices
        temp = FindUSB3DAQ()
        If (temp > 0) Then
            ' Add each available device index to the combo box
            For i = 0 To temp - 1
                comboboxDeviceIndex.Items.Add(i)
            Next
            ' Select the first device index in the combo box
            comboboxDeviceIndex.SelectedIndex = 0
            ' Enable the start button
            btnStart.Enabled = True
        Else
            ' Disable the start button if no device is found
            btnStart.Enabled = False
        End If

        ' Disable the stop button initially
        btnStop.Enabled = False

    End Sub

    Private Sub btnStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStart.Click

        Dim i As Integer
        Dim j As Integer
        Dim temp As Integer
        Dim SN(8) As Byte

        ' Generate a waveform consisting of 1000 random numbers between 0 and 1V and store it in WaveData
        For i = 0 To 999
            WaveData(i) = Rnd()
        Next

        ' Get the selected device index from the combo box
        DevIndex = comboboxDeviceIndex.SelectedIndex

        ' Open the device with the selected index
        temp = USB3OpenDevice(DevIndex)

        ' Get and display the device serial number
        temp = USB3GetDeviceSN(DevIndex, SN(0))
        textboxSN.Text = System.Text.Encoding.ASCII.GetString(SN)

        '-----------------------------------------
        ' Ao Wave related settings

        ' Clear the trigger flag of Ao_0
        temp = SetUSB3ClrAoTrigger(DevIndex, 0)

        ' Clear the hardware FIFO space of Ao_0
        temp = SetUSB3ClrAoFifo(DevIndex, 0)

        ' Set the output mode of Ao_0 to computer-buffered infinite-length non-looping waveform output mode
        temp = SetUSB3AoSampleMode(DevIndex, 0, 0)

        ' Set the sampling period of Ao_0 input to 10000 ns (10us), with a sampling rate of 100kHz
        temp = SetUSB3AoSampleRate(DevIndex, 0, 1000)

        ' Set the trigger source of Ao_0 to software trigger
        temp = SetUSB3AoTrigSource(DevIndex, 0, 0)

        ' Set the sampling clock of Ao_0 to the internal Ao sampling clock
        temp = SetUSB3AoConvSource(DevIndex, 0, 0)

        ' Download the 1000-point waveform WaveData to the hardware FIFO
        temp = SetUSB3AoDataFifo(DevIndex, 0, WaveData(0), 1000)

        ' Disable the start button
        btnStart.Enabled = False
        ' Enable the stop button
        btnStop.Enabled = True
        ' Initialize the number of rounds
        Rounds = 0
        ' Set the running flag to true
        bRun = True

        ' Set the timer interval and enable it
        Timer1.Interval = 100
        Timer1.Enabled = True

        ' Trigger Ao_0 with software
        temp = SetUSB3AoSoftTrig(DevIndex, 0)

        ' Create a thread to continuously generate new data and download it to the acquisition card hardware FIFO
        threadDownload = New Threading.Thread(AddressOf DownloadWaveData)
        threadDownload.Start()

    End Sub

    Private Sub DownloadWaveData()

        Dim temp As Integer
        Dim i As Integer

        ' Loop while the device is running
        Do While bRun = True

            ' Update the waveform data
            For i = 0 To 999
                WaveData(i) = Rnd()
            Next

            ' Download the updated 1000-point waveform WaveData to the hardware FIFO
            temp = SetUSB3AoDataFifo(DevIndex, 0, WaveData(0), 1000)

            ' Increment the number of rounds
            Rounds = Rounds + 1

            ' Since 1000 points of data can be output for 10ms, delay for 5ms before generating new data and downloading to ensure continuous output waveform
            Sleep(5)
        Loop

    End Sub

    Private Sub btnStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop.Click

        CloseDevice()
        btnStart.Enabled = True
        btnStop.Enabled = False

    End Sub

    Private Sub CloseDevice()

        Dim temp As Integer

        If bRun = True Then
            ' Set the running flag to false
            bRun = False
            ' Wait for the download thread to finish
            threadDownload.Join()

            ' Clear the Ao_0 trigger flag
            temp = SetUSB3ClrAoTrigger(DevIndex, 0)

            ' Clear the Ao_0 hardware FIFO
            temp = SetUSB3ClrAoFifo(DevIndex, 0)

            ' Close the device
            temp = USB3CloseDevice(DevIndex)
        End If

    End Sub

    Private Sub Form1_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing

        CloseDevice()

    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick

        ' Display the number of rounds
        textboxRounds.Text = Rounds

    End Sub
End Class