﻿Public Class Form1

    ' Declare the Sleep function from Kernel32.dll
    Private Declare Sub Sleep Lib "Kernel32" (ByVal dwMilliseconds As Integer)

    ' Array of checkboxes for AI channels (AI functionality is not fully utilized in this code)
    Dim checkboxAi(0 To 23) As CheckBox
    ' Array of textboxes for AI channel values (AI functionality is not fully utilized in this code)
    Dim textboxAiValue(0 To 23) As TextBox
    ' Array of checkboxes for DI channels (not used in this code)
    Dim checkboxDi(0 To 3) As CheckBox
    ' Array of checkboxes for DO channels
    Dim checkboxDo(0 To 3) As CheckBox

    ' Index of the selected device
    Dim DevIndex As Integer
    ' Flag indicating whether the device is running
    Dim bRun As Boolean

    ' Array to store waveform data
    Dim WaveData(1000) As UInteger
    ' Variable to keep track of the number of rounds
    Dim Rounds As ULong
    ' Thread for downloading waveform data
    Dim threadDownload As Threading.Thread


    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        Dim temp As Integer
        Dim i As Integer

        ' Find the USB 3 DAQ device and get the number of available devices
        temp = FindUSB3DAQ()
        If (temp > 0) Then
            ' Add each available device index to the combobox
            For i = 0 To temp - 1
                comboboxDeviceIndex.Items.Add(i)
            Next
            ' Select the first device index in the combobox
            comboboxDeviceIndex.SelectedIndex = 0
            ' Enable the start button
            btnStart.Enabled = True
        Else
            ' Disable the start button if no device is found
            btnStart.Enabled = False
        End If

        ' Initially disable the stop button
        btnStop.Enabled = False

    End Sub

    Private Sub btnStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStart.Click

        Dim i As Integer
        Dim j As Integer
        Dim temp As Integer
        Dim SN(8) As Byte

        ' Generate a waveform consisting of 1000 random numbers and store it in WaveData
        ' The lower 4 bits of each WaveData correspond to 4 DO channels
        For i = 0 To 999
            WaveData(i) = Rnd() * 15
        Next

        ' Get the selected device index from the combobox
        DevIndex = comboboxDeviceIndex.SelectedIndex

        ' Open the device with the selected index
        temp = USB3OpenDevice(DevIndex)

        ' Get and display the device serial number
        temp = USB3GetDeviceSN(DevIndex, SN(0))
        textboxSN.Text = System.Text.Encoding.ASCII.GetString(SN)

        '-----------------------------------------
        ' DO Wave related settings

        ' Select the channels for the waveform output
        '&HF each bit corresponds to a channel, and &HF means 4 channels are selected
        temp = SetUSB3DoWaveCtrl(DevIndex, &HF)

        ' Clear the trigger flag of DO
        temp = SetUSB3ClrDoTrigger(DevIndex)

        ' Clear the hardware FIFO space of DO
        temp = SetUSB3ClrDoFifo(DevIndex)

        ' Set the output mode of DO to computer-buffered infinite-length non-looping waveform output mode
        temp = SetUSB3DoSampleMode(DevIndex, 0)

        ' Set the sampling period of DO input to 10000 ns (10us), with a sampling rate of 100kHz
        temp = SetUSB3DoSampleRate(DevIndex, 1000)

        ' Set the trigger source of DO to software trigger
        temp = SetUSB3DoTrigSource(DevIndex, 0)

        ' Set the sampling clock of DO to the internal DO sampling clock
        temp = SetUSB3DoConvSource(DevIndex, 0)

        ' Download the 1000-point waveform WaveData to the hardware FIFO
        temp = SetUSB3DoDataFifo(DevIndex, WaveData(0), 1000)

        ' Disable the start button
        btnStart.Enabled = False
        ' Enable the stop button
        btnStop.Enabled = True
        ' Initialize the number of rounds
        Rounds = 0
        ' Set the running flag to true
        bRun = True

        ' Set the timer interval and enable it
        Timer1.Interval = 100
        Timer1.Enabled = True

        ' Trigger DO with software
        temp = SetUSB3DoSoftTrig(DevIndex)

        ' Create a thread to continuously generate new data and download it to the acquisition card hardware FIFO
        threadDownload = New Threading.Thread(AddressOf DownloadWaveData)
        threadDownload.Start()

    End Sub

    Private Sub DownloadWaveData()

        Dim temp As Integer
        Dim i As Integer

        ' Loop while the device is running
        Do While bRun = True

            ' Update the waveform data
            For i = 0 To 999
                WaveData(i) = Rnd() * 15
            Next

            ' Download the updated 1000-point waveform WaveData to the hardware FIFO
            temp = SetUSB3DoDataFifo(DevIndex, WaveData(0), 1000)

            ' Increment the number of rounds
            Rounds = Rounds + 1

            ' Since 1000 points of data can be output for 10ms, delay for 5ms before generating new data and downloading to ensure continuous output waveform
            Sleep(5)
        Loop

    End Sub

    Private Sub btnStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop.Click

        CloseDevice()
        ' Enable the start button
        btnStart.Enabled = True
        ' Disable the stop button
        btnStop.Enabled = False

    End Sub

    Private Sub CloseDevice()

        Dim temp As Integer

        If bRun = True Then
            ' Set the running flag to false
            bRun = False
            ' Wait for the download thread to finish
            threadDownload.Join()

            ' Clear the trigger flag of DO
            temp = SetUSB3ClrDoTrigger(DevIndex)

            ' Clear the hardware FIFO space of DO
            temp = SetUSB3ClrDoFifo(DevIndex)

            ' Close the device
            temp = USB3CloseDevice(DevIndex)
        End If

    End Sub

    Private Sub Form1_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing

        CloseDevice()

    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick

        ' Display the number of rounds
        textboxRounds.Text = Rounds

    End Sub
End Class